<?php

namespace Assetise\Scheduler\Http\Controllers;

use App\Http\Controllers\ActionController as BaseController;
use Exception;
use Assetise\Scheduler\Actions\SchedulerAction;
use Assetise\Scheduler\Http\Requests\SchedulerActionRequest;
use Assetise\Scheduler\Http\Resources\SchedulerResource;
use Assetise\Scheduler\Models\Scheduler;

/**
 * Admin web controller class.
 */
class SchedulerActionController extends BaseController
{

    /**
     * Action controller function for scheduler.
     *
     * @param Model $model
     * @param action next action for the model.
     *
     * @return Response
     */
    public function __invoke(SchedulerActionRequest $request, Scheduler $scheduler, $action)
    {
        try {
            $request = $request->all();
            $scheduler = SchedulerAction::run($action, $scheduler, $request);

            $data = new SchedulerResource($scheduler);
            return $this->response->message(trans("messages.success.{$action}", ['Module' => trans('scheduler::scheduler.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('scheduler/scheduler/' . $scheduler->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('scheduler/scheduler/' . $scheduler->getRouteKey()))
                ->redirect();
        }
    }

    public function notify(){
        
    }
}
