<?php

namespace Assetise\Scheduler\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Exception;
use Assetise\Scheduler\Http\Requests\SchedulerActionsRequest;
use Assetise\Scheduler\Actions\SchedulerActions;

/**
 * Admin web controller class.
 */
class SchedulerActionsController extends BaseController
{

    /**
     * Action controller function for scheduler.
     *
     * @param Model $model
     * @param action next action for the model.
     *
     * @return Response
     */
    public function __invoke(SchedulerActionsRequest $request, $action)
    {
        try {
            $scheduler = SchedulerActions::run($request, $action);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('scheduler::scheduler.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('scheduler/scheduler/' . $action))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('scheduler/scheduler/' . $action))
                ->redirect();
        }
    }
}
