<?php

namespace Assetise\Scheduler\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Scheduler\Events\SchedulerAction as SchedulerActionEvent;
use Assetise\Scheduler\Notifications\SchedulerAction as SchedulerActionNotification;
use Litepie\Actions\Concerns\AsAction;

class SchedulerAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the SchedulerActionEvent.
     *
     * @param   SchedulerActionEvent  $event
     * @return mixed
     */
    public function handle(SchedulerActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $scheduler.
     *
     * @param   SchedulerActionEvent  $event
     * @return void
     */
    public function create(SchedulerActionEvent $event)
    {
        $client = $event->scheduler->client;
        Notification::send($client, new SchedulerActionNotification($event));
    }

    /**
     * Handle the SchedulerActionEvent as a listener.
     *
     * @param   SchedulerActionEvent  $event
     * @return mixed
     */
    public function asListener(SchedulerActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   SchedulerActionEvent  $event
     * @return bool
     */
    private function isAllowed(SchedulerActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
