<?php

namespace Assetise\Scheduler\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Scheduler\Events\SchedulerWorkflow as SchedulerWorkflowEvent;
use Assetise\Scheduler\Notifications\SchedulerWorkflow as SchedulerWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class SchedulerWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(SchedulerWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $scheduler is submitted.
     *
     * @param SchedulerWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(SchedulerWorkflowEvent $event)
    {
        $client = $event->scheduler->client;
        Notification::send($client, new SchedulerWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $scheduler is published.
     *
     * @param SchedulerWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(SchedulerWorkflowEvent $event)
    {
        $client = $event->scheduler->client;
        Notification::send($client, new SchedulerWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $scheduler is approved.
     *
     * @param SchedulerWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(SchedulerWorkflowEvent $event)
    {
        $client = $event->scheduler->client;
        Notification::send($client, new SchedulerWorkflowNotification($event));
    }

    /**
     * Handles the $scheduler workflow event as a listener.
     *
     * @param SchedulerWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(SchedulerWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
