<?php

namespace Assetise\Scheduler\Providers;

use Illuminate\Support\ServiceProvider;
use Assetise\Scheduler\Schedulers;

class SchedulerServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = true;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'scheduler');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'scheduler');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerFacade();
        $this->registerCommands();

        $this->app->register(\Assetise\Scheduler\Providers\AuthServiceProvider::class);
        $this->app->register(\Assetise\Scheduler\Providers\RouteServiceProvider::class);
        $this->app->register(\Assetise\Scheduler\Providers\ActionServiceProvider::class);
        $this->app->register(\Assetise\Scheduler\Providers\EventServiceProvider::class);
        $this->app->register(\Assetise\Scheduler\Providers\WorkflowServiceProvider::class);
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->bind('assetise.scheduler', function($app)
        {
            return $this->app->make(Scheduler::class);
        });
    }

    /**
     * Merges user's and scheduler's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'assetise.scheduler'
        );
        
        
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/scheduler.php', 'assetise.scheduler.scheduler'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                \Assetise\Scheduler\Commands\Scheduler::class,
            ]);
        }
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['assetise.scheduler'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/' => config_path('assetise/scheduler')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/scheduler')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/scheduler')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
